/** ProgramName
  *     An AviSynth plug-in to retrieve the name of the calling program.
  *
  * Last modified: 2009-03-10
  *
  * Copyright (c) 2009 J.M. Roelofs
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
  * met:
  *
  * * Redistributions of source code must retain the above copyright
  *   notice, this list of conditions and the following disclaimer.
  *
  * * Redistributions in binary form must reproduce the above copyright
  *   notice, this list of conditions and the following disclaimer in the
  *   documentation and/or other materials provided with the distribution.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
  * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */

#include <vector>
#include <string>

#include <cstdlib>
#include <cstring>
#include <cassert>

#include "windows.h"
#include "tchar.h"
#include "avisynth.h"

#include "ScopeGuard.h"
#include "atlconv.h"

typedef std::basic_string<TCHAR> tstring;


// FUNCTION DEFINITIONS ------------------------------------------------

/** GetErrorMessage
  *
  *     Retrieves the system error message associated with a given error
  *     code.
  *
  * PARAMETERS:
  *     err : The error code.
  *
  * RETURNS:
  *     The error message.
  */
static tstring GetErrorMessage(DWORD err)
{
    tstring::value_type* p;
    if (FormatMessage(  FORMAT_MESSAGE_ALLOCATE_BUFFER
                      | FORMAT_MESSAGE_IGNORE_INSERTS
                      | FORMAT_MESSAGE_FROM_SYSTEM,
                      NULL, err, 0, reinterpret_cast<tstring::value_type*>(&p), 0, NULL)
        != 0)
    {
        ON_BLOCK_EXIT(LocalFree, p);
        return tstring(p);
    }
    else
    {
        return tstring();
    }
}


/** GetScriptName
  *
  *     Retrieves the name of the calling program from the commandline.
  *
  *     Callback passed to the scripting environment.
  *
  * PARAMETERS:
  *     args             - (unused)
  *     IN/OUT userDataP - (unused)
  *     IN/OUT envP      - Pointer to the AviSynth scripting environment.
  *
  * RETURNS:
  *     The name of the current avisynth-file.
  */
static AVSValue __cdecl GetProgramName(AVSValue /* args */, void* /* userDataP */, IScriptEnvironment* envP)
{
	USES_CONVERSION;
	LPWSTR *szArglist;
	int nArgs;
	char* val;
	char* shortval;

	szArglist = CommandLineToArgvW(GetCommandLineW(), &nArgs);
	if( NULL == szArglist )
	{
		envP->ThrowError("GetAvsName: Commandline could not be read: ",
                         GetErrorMessage(GetLastError()).c_str());
	} 
	val = W2T(szArglist[0]);

	shortval=strrchr(val,'\\');
			if (NULL != shortval)
			{				
				val = shortval + 1;
			}

	const char* copy = envP->SaveString(val, strlen(val));
    return AVSValue(copy);
}

/** AvisynthPluginInit2
  *
  *     AviSynth 2.5x plug-in entry-point.
  *
  * PARAMETERS:
  *     IN/OUT envP - pointer to the AviSynth scripting environment
  *
  * RETURNS:
  *     the version string for this plug-in
  *
  * SIDE EFFECTS:
  *     adds functions to the AviSynth scripting environment
  */
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* envP)
{
    envP->AddFunction("GetProgramName", "", GetProgramName, NULL);

    return "ProgramName v0.1.0 [" __DATE__ "]\nCopyright (c) 2009 J.M. Roelofs";
}
